package cz.sk1x1;

import okhttp3.*;
import org.json.JSONArray;
import org.json.JSONObject;
import org.openqa.selenium.*;
import org.openqa.selenium.firefox.FirefoxDriver;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;


public class Main {
    private final static boolean ENABLE_CONSOLE_LOGGING = false;

    /* Program arguments */
    /** Base address of Drupal site */
    static String baseAddress = "";
    /** Relative address to search form */
    static String formAddress = "";
    /** Name of search field */
    static String fieldName = "";
    /** Login name */
    static String loginName = "";
    /** Login password */
    static String loginPassword = "";

    /** List of similar words from API */
    static List<String> similarWords = null;

    /** Similar words API address */
    static String apiAddress = "";

    /** Testing driver */
    static WebDriver driver;

    /* Constants */
    /** Config form address */
    final static String configAddress = "admin/config/search/search_api_solr_similar_words";
    /** Relative address of login form */
    final static String loginAddress = "user/login";
    /** Search word */
    final static String searchedWord = "hrad";


    public static void main(String[] args) {
        if(!ENABLE_CONSOLE_LOGGING) {
            System.setProperty(FirefoxDriver.SystemProperty.BROWSER_LOGFILE, "/dev/null");
        }

        if(args.length < 5) {
            System.out.println("Not enough arguments. Usage of program:");
            System.out.println("  test.jar BASE_ADDRESS FORM_ADDRESS FIELD_NAME LOGIN_NAME LOGIN_PASSWORDS");
            System.out.println("");
            System.out.println("BASE_ADDRESS      Base address of Drupal site");
            System.out.println("FORM_ADDRESS      Relative address to search form");
            System.out.println("FIELD_NAME        Name of search field");
            System.out.println("LOGIN_NAME        Login name");
            System.out.println("LOGIN_PASSWORDS   Login password");
            System.exit(1);
        }
        baseAddress = args[0];
        formAddress = args[1];
        fieldName = args[2];
        loginName = args[3];
        loginPassword = args[4];

        System.setProperty("webdriver.gecko.driver", "G:\\Downloads\\geckodriver-v0.27.0-win64\\geckodriver.exe");
        //System.setProperty("webdriver.chrome.driver", "G:\\Downloads\\chromedriver_win32\\chromedriver.exe");

        driver = new FirefoxDriver();
        runTests();
        driver.quit();
    }

    /**
     * Run tests in right order.
     */
    private static void runTests() {
        login();
        setUp();
        getSimilarWords();
        checkSimilarWordsInForm();
    }

    /**
     * Login user.
     */
    private static void login() {
        try {
            testStart("login");

            driver.get(baseAddress + loginAddress);

            driver.findElement(By.name("name")).sendKeys(loginName);
            driver.findElement(By.name("pass")).sendKeys(loginPassword);
            driver.findElement(By.id("edit-submit")).click();

            testOk();
        }
        catch (NoSuchElementException e) {
            testFailed(e.getMessage());
        }
    }

    /**
     * Opens configuration form, sets number of shown similar words and copy API address.
     */
    private static void setUp() {
        try {
            testStart("setUp");

            driver.get(baseAddress + configAddress);

            // ziskam adresu API na podobna slova
            WebElement element = driver.findElement(By.name("search_api_solr_similar_words_address"));
            apiAddress = element.getAttribute("value");

            // nastavim zobrazeni 10 podobnych slov
            driver.findElement(By.name("search_api_solr_similar_words_max")).sendKeys(Keys.chord(Keys.CONTROL, "A"));
            driver.findElement(By.name("search_api_solr_similar_words_max")).sendKeys("10");
            driver.findElement(By.id("edit-submit")).click();

            testOk();
        }
        catch (NoSuchElementException e) {
            testFailed(e.getMessage());
        }
    }

    /**
     * Gets similar words from API. Used for check if form contains correct words.
     */
    private static void getSimilarWords() {
        String responseJson = "";
        String content =
                "{" +
                "\"queryLanguage\": \"cs\", " +
                "\"queryWord\": \"" + searchedWord + "\", " +
                "\"targetLanguage\": \"cs\" " +
                "}";
        OkHttpClient client = new OkHttpClient();
        RequestBody body = RequestBody.create(MediaType.get("application/json; charset=utf-8"),
                content);
        Request request = new Request.Builder()
                .url(apiAddress)
                .post(body)
                .build();
        try (Response response = client.newCall(request).execute()) {
            responseJson = response.body().string();
        } catch (IOException e) {
            e.printStackTrace();
        }
        similarWords = new ArrayList<>();
        JSONObject json = new JSONObject(responseJson);
        JSONArray words = json.getJSONArray("words");

        for(int i = 0; i < words.length(); i++) {
            similarWords.add(words.getJSONObject(0).getString("word"));
        }
    }

    /**
     * Check similar words in search form.
     */
    private static void checkSimilarWordsInForm() {
        try {
            testStart("checkSimilarWordsInForm");

            driver.get(baseAddress + formAddress);

            // ziskam adresu API na podobna slova
            driver.findElement(By.name(fieldName)).sendKeys(searchedWord);
            driver.findElement(By.className("form-submit")).click();

            List<WebElement> checkboxLabels = driver.findElements(By.xpath("//div[input]/label"));
            for(String word : similarWords) {
                boolean found = false;
                for(WebElement element : checkboxLabels) {
                    if(element.getText().equals(word)) {
                        found = true;
                        break;
                    }
                }

                if(!found) {
                    testFailed("Similar word \"" + word + "\" was not found in search form");
                    break;
                }
            }

            testOk();
        }
        catch (NoSuchElementException e) {
            testFailed(e.getMessage());
        }
    }

    /**
     * Print test start.
     *
     * @param testName Test name
     */
    private static void testStart(String testName) {
        System.out.println("Test " + testName + " started");
    }

    /**
     * Print test failed with message.
     *
     * @param error Error message
     */
    private static void testFailed(String error) {
        if(error != null && !error.isEmpty()) {
            System.err.println("Test FAILED: " + error);
        }
        else {
            System.err.println("Test FAILED");
        }
    }

    /**
     * Print test failed.
     */
    private static void testFailed() {
        testFailed("");
    }

    /**
     * Print test is OK with message.
     *
     * @param message Message
     */
    private static void testOk(String message) {
        if(message != null && !message.isEmpty()) {
            System.out.println("Test OK (" + message + ")");
        }
        else {
            System.err.println("Test OK");
        }
    }

    /**
     * Print test is OK.
     */
    private static void testOk() {
        System.out.println("Test OK");
    }
}